@@ -1,7 +1,10 @@
+from __future__ import annotations
+from typing import Callable
 import torch
 import lantern
 
 from perceptor.transforms.clamp_with_grad import clamp_with_grad
+from . import diffusion_space
 
 
 class Predictions(lantern.FunctionalBase):
@@ -10,12 +13,14 @@ class Predictions(lantern.FunctionalBase):
     predicted_noise: lantern.Tensor.dims("NCHW")
     schedule_alphas: lantern.Tensor
     schedule_sigmas: lantern.Tensor
+    encode: Callable[[lantern.Tensor.dims("NCHW")], lantern.Tensor.dims("NCHW")]
+    decode: Callable[[lantern.Tensor.dims("NCHW")], lantern.Tensor.dims("NCHW")]
 
     @property
     def device(self):
         return self.predicted_noise.device
 
-    def indices(self, indices):
+    def indices(self, indices) -> lantern.Tensor:
         if isinstance(indices, float) or isinstance(indices, int):
             indices = torch.as_tensor(indices)
         if indices.ndim == 0:
@@ -24,31 +29,35 @@ class Predictions(lantern.FunctionalBase):
             raise ValueError("indices must be a scalar or a 1D tensor")
         return indices.long().to(self.device)
 
-    def alphas(self, indices):
+    def alphas(self, indices) -> lantern.Tensor:
         return self.schedule_alphas[self.indices(indices)][:, None, None, None].to(
             self.device
         )
 
-    def sigmas(self, indices):
+    def sigmas(self, indices) -> lantern.Tensor:
         return self.schedule_sigmas[self.indices(indices)][:, None, None, None].to(
             self.device
         )
 
     @property
-    def from_alphas(self):
+    def from_alphas(self) -> lantern.Tensor:
         return self.alphas(self.from_indices)
 
     @property
-    def from_sigmas(self):
+    def from_sigmas(self) -> lantern.Tensor:
         return self.sigmas(self.from_indices)
 
     @property
-    def denoised_latents(self):
+    def denoised_latents(self) -> lantern.Tensor:
         return (
             self.from_diffused_latents - self.from_sigmas * self.predicted_noise
         ) / self.from_alphas.clamp(min=1e-7)
 
-    def step(self, to_indices, eta=0.0):
+    @property
+    def denoised_images(self) -> lantern.Tensor:
+        return self.decode(self.denoised_latents)
+
+    def step(self, to_indices, eta=0.0) -> lantern.Tensor:
         """
         Reduce noise level to `to_indices`
 
@@ -90,7 +99,7 @@ class Predictions(lantern.FunctionalBase):
 
         # TODO: do not need to calculate denoised latents? this could introduce errors?
 
-    def correction(self, previous: "Predictions") -> "Predictions":
+    def correction(self, previous: Predictions) -> Predictions:
         # k-diffusion has alphas=1 always so this should not work
         # corrected_diffused_xs = (
         #     previous.from_diffused_xs
@@ -110,7 +119,7 @@ class Predictions(lantern.FunctionalBase):
             (self.denoised_latents + previous.denoised_latents) / 2
         )
 
-    def reverse_step(self, to_indices):
+    def reverse_step(self, to_indices) -> lantern.Tensor:
         if (torch.as_tensor(self.from_indices) > torch.as_tensor(to_indices)).any():
             raise ValueError("from_indices must be less than to_indices")
 
@@ -129,7 +138,7 @@ class Predictions(lantern.FunctionalBase):
         return self.denoised_latents * to_alphas + self.predicted_noise * to_sigmas
         # TODO: do not need to calculate denoied latents? this could introduce errors?
 
-    def resample(self, resample_indices):
+    def resample(self, resample_indices) -> lantern.Tensor:
         """
         Harmonizing resampling from https://github.com/andreas128/RePaint
         """
@@ -138,7 +147,7 @@ class Predictions(lantern.FunctionalBase):
             + self.resample_noise(resample_indices) * self.from_sigmas
         )
 
-    def resample_noise(self, resample_indices):
+    def resample_noise(self, resample_indices) -> lantern.Tensor:
         if (
             torch.as_tensor(self.from_indices) < torch.as_tensor(resample_indices)
         ).any():
@@ -151,7 +160,7 @@ class Predictions(lantern.FunctionalBase):
         )  # fmt: skip
         return resampled_noise_sigma / self.from_sigmas
 
-    def noisy_reverse_step(self, to_indices):
+    def noisy_reverse_step(self, to_indices) -> lantern.Tensor:
         to_alphas, to_sigmas = self.alphas(to_indices), self.sigmas(to_indices)
 
         noise_sigma = self.from_sigmas * self.predicted_noise + (
@@ -160,7 +169,7 @@ class Predictions(lantern.FunctionalBase):
 
         return self.denoised_latents * to_alphas + noise_sigma
 
-    def guided(self, guiding, guidance_scale=0.5, clamp_value=1e-6) -> "Predictions":
+    def guided(self, guiding, guidance_scale=0.5, clamp_value=1e-6) -> Predictions:
         return self.replace(
             predicted_noise=self.predicted_noise
             + guidance_scale
@@ -169,34 +178,36 @@ class Predictions(lantern.FunctionalBase):
             / clamp_value
         )
 
-    def dynamic_threshold(self, quantile=0.95) -> "Predictions":
+    def dynamic_threshold(self, quantile=0.95) -> Predictions:
         """
         Thresholding heuristic from imagen paper
         """
+        denoised_xs = diffusion_space.encode(self.decode(self.denoised_latents))
         dynamic_threshold = torch.quantile(
-            self.denoised_latents.flatten(start_dim=1).abs(), quantile, dim=1
+            denoised_xs.flatten(start_dim=1).abs(), quantile, dim=1
         ).clamp(min=1.0)
-        denoised_latents = (
+        denoised_xs = (
             clamp_with_grad(
-                self.denoised_latents,
+                denoised_xs,
                 -dynamic_threshold,
                 dynamic_threshold,
             )
-            # / dynamic_threshold
-            # imagen's dynamic thresholding divides by threshold but this makes the images gray
+            / dynamic_threshold
+        )
+        return self.forced_denoised_latents(
+            self.encode(diffusion_space.decode(denoised_xs))
         )
-        return self.forced_denoised_latents(denoised_latents)
 
-    def forced_denoised_latents(self, denoised_latents) -> "Predictions":
+    def forced_denoised_latents(self, denoised_latents) -> Predictions:
         predicted_noise = (
             self.from_diffused_latents - denoised_latents * self.from_alphas
         ) / self.from_sigmas.clamp(min=1e-7)
         return self.replace(predicted_noise=predicted_noise)
 
-    def forced_predicted_noise(self, predicted_noise) -> "Predictions":
+    def forced_predicted_noise(self, predicted_noise) -> Predictions:
         return self.replace(predicted_noise=predicted_noise)
 
-    def wasserstein_distance(self):
+    def wasserstein_distance(self) -> lantern.Tensor:
         sorted_noise = self.predicted_noise.flatten(start_dim=1).sort(dim=1)[0]
         n = sorted_noise.shape[1]
         margin = 0.5 / n
@@ -204,7 +215,7 @@ class Predictions(lantern.FunctionalBase):
         expected_noise = torch.distributions.Normal(0, 1).icdf(points)
         return (sorted_noise - expected_noise[None].to(sorted_noise)).abs().mean()
 
-    def wasserstein_square_distance(self):
+    def wasserstein_square_distance(self) -> lantern.Tensor:
         sorted_noise = self.predicted_noise.flatten(start_dim=1).sort(dim=1)[0]
         n = sorted_noise.shape[1]
         margin = 0.5 / n
@@ -213,8 +224,8 @@ class Predictions(lantern.FunctionalBase):
         return (sorted_noise - expected_noise[None].to(sorted_noise)).square().mean()
 
     def classifier_free_guidance(
-        self, positive_predictions: "Predictions", guidance_scale=7.0
-    ) -> "Predictions":
+        self, positive_predictions: Predictions, guidance_scale=7.0
+    ) -> Predictions:
         return self.replace(
             predicted_noise=self.predicted_noise
             + (positive_predictions.predicted_noise - self.predicted_noise)
